/**
 In this lesson we will learn more about memory management.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
  If we want to write a function that returns a pointer to something, e.g. an array, then we must pay careful attention to the lifetime of the variable.
 */
int * getValues(){
  int values[] = {1, 2, 3, 4, 5};
  return values;
  /**
   This code is very dangerous because the array is only valid within this function.
   The pointer is returned, but as it points to a locally defined array, the memory at that location might have been overwritten by the time the pointer is accessed.
   Therefore this code can create serious errors that are not easy to find.
  */
}

/*
Generally, all variables are defined `on the stack'. This is, as the name implies, organised such that when entering a new scope (or block) all variables are placed on the stack. Upon leaving the block, the next blocks addresses will be saved on the stack. Function calls are also placed on the stack.

We can also allocate memory next to the stack on what is called `the heap'. The heap is completely managed by the user with no automatic assistance. Thus the user must request to free any memory they allocate (at the end of the program, the memory is almost always automically released).
*/

/*
  An example of how to request and return memory from a function.
 */
float * allocateArray(int length){
  // Malloc requests a memory block of a certain size, we want `length` elements which are all of the size of a float. The compiler provides the command `sizeof` to determine this.
  return malloc(length * sizeof(float));
}

int main(int argc, char ** argv){
  const long arraySize = atol(argv[1]);

  float * const array = allocateArray(arraySize);

  for(int i=0; i < arraySize; i++){
    array[i] = i;
  }

  // With the command `free` we release the memory again for other use.
  free(array);

  printf("Completed!\n");

  /*
    When writing a program that access memory via pointers, several errors can occur:
    - you access memory that (presumably) does not belong to you.
  */
  array[arraySize * 10] = 0;

  // - you access memory that is read-only (for example machine code which is stored in memory)

  // the memory accessed might look like it is owned, but is possibly actually a different variable.
  array[-1] = 0;
  array[arraySize + 1] = 0;

  // you access memory which has already been freed.
  array[0] = 1; // is this code valid?

  // if you are lucky, the program will crash with a memory error (Segmentation Fault / Segfault).

  // if you are unlucky, you cause unexpected changes such as altering a loop variable preventing it from terminating.
  // the resulting errors are very different from each other and can be difficult to debug because of the cause of the error being in a different place to where the effect of the error occurs.
  // The error might also occur intermittently, only effecting some runs of the program, since memory areas are arbitrarily assigned.

  // The tool `Valgrin` allows for debugging memory errors, and is something that you should look into.
  // valgrind ./badcode
  // Note: valgrind will increase the runtime of a program.

  // Typically, you should always use Valgrind first, and then once you have resolved valgrinds errors use gdb or another debugger.

  return 0;
}